import	os
import	unittest
from	pygwarts.magical.philosophers_stone	import Transmutable
from	pygwarts.irma.contrib				import LibraryContrib
from	pygwarts.tests.magical				import MagicalTestCase








class StoneCase(MagicalTestCase):
	def test_no_loggy(self):

		class Stone(Transmutable):
			class Mutate(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		self.test_case = Stone()
		self.assertEqual(self.test_case.Mutate(), "I am part of mutable chain!")
		self.assertEqual(self.test_case.loggy.info("I am root"), NotImplemented)


		class Stone(Transmutable):	pass
		self.test_case = Stone()
		self.assertEqual(self.test_case.loggy.info("I am root"), NotImplemented)




	def test_escalated_loggy(self):

		escalated_loggy = str(self.MAGICAL_ROOT /"escalated_loggy.loggy")
		self.make_loggy_file(escalated_loggy)

		class Stone(Transmutable):
			class Mutate(Transmutable):

				class loggy(LibraryContrib):

					handler		= escalated_loggy
					init_name	= "escalated_loggy-1"

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		self.test_case = Stone()
		self.assertEqual(self.test_case.Mutate(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		loggy = []

		self.test_case.loggy.close()
		with open(escalated_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@escalated_loggy-1 INFO : I am part of mutable chain!",
				"@escalated_loggy-1 INFO : I am root",
			]
		)


		self.make_loggy_file(escalated_loggy)
		class Stone(Transmutable):
			class Mutate(Transmutable):

				class loggy(LibraryContrib):

					handler			= escalated_loggy
					init_name		= "escalated_loggy-2"
					force_handover	= True

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		self.test_case = Stone()
		self.assertEqual(self.test_case.Mutate(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		loggy = []

		self.test_case.loggy.close()
		with open(escalated_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@escalated_loggy-2-Stone.Mutate INFO : I am part of mutable chain!",
				"@escalated_loggy-2-Stone INFO : I am root",
			]
		)


		class Stone(Transmutable):
			class Mutate(Transmutable):

				class loggy(LibraryContrib):

					handler		= escalated_loggy
					init_name	= "escalated_loggy-3"
					init_level	= 10

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		self.make_loggy_file(escalated_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case.Mutate(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		loggy = []

		self.test_case.loggy.close()
		with open(escalated_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@escalated_loggy-3-Stone.Mutate INFO : I am part of mutable chain!",
				"@escalated_loggy-3-Stone INFO : I am root",
			]
		)
		if	os.path.isfile(escalated_loggy): os.remove(escalated_loggy)








	def test_name_loggy(self):

		name_loggy = str(self.MAGICAL_ROOT /"name_loggy.loggy")
		self.make_loggy_file(name_loggy)

		class Stone(Transmutable):
			class shoggy(LibraryContrib):

				handler		= name_loggy
				init_name	= "name_loggy-1"

			class Mutate(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		self.test_case = Stone()
		self.assertEqual(self.test_case.Mutate(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		loggy = []

		self.test_case.loggy.close()
		with open(name_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@name_loggy-1 INFO : I am part of mutable chain!",
				"@name_loggy-1 INFO : I am root",
			]
		)


		class Stone(Transmutable):

			class Mutate(Transmutable):
				class shoggy(LibraryContrib):

					handler		= name_loggy
					init_name	= "name_loggy-2"

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		self.make_loggy_file(name_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case.Mutate(), "I am part of mutable chain!")
		self.assertNotEqual(self.test_case.loggy.info("I am root"), NotImplemented)
		loggy = []

		self.test_case.loggy.close()
		with open(name_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@name_loggy-2 INFO : I am part of mutable chain!",
				"@name_loggy-2 INFO : I am root",
			]
		)
		if	os.path.isfile(name_loggy): os.remove(name_loggy)








	def test_mutual_loggy(self):

		mutual_loggy = str(self.MAGICAL_ROOT /"mutual_loggy.loggy")
		self.make_loggy_file(mutual_loggy)

		class StoneOne(Transmutable):	pass
		class StoneTwo(Transmutable):	pass

		logger = LibraryContrib(handler=mutual_loggy, init_name="mutual_loggy", force_handover=True)
		stone_one = StoneOne(logger)
		stone_two = StoneTwo(logger)

		stone_one.loggy.info("OOH")
		stone_two.loggy.info("EEH")
		stone_one.loggy.info("OOH")
		stone_two.loggy.info("AH")
		stone_one.loggy.info("AH")
		stone_two.loggy.info("TING")
		stone_one.loggy.info("TANG")
		stone_two.loggy.info("WALLA")
		stone_one.loggy.info("WALLA")
		stone_two.loggy.info("BING")
		stone_one.loggy.info("BANG")
		loggy = []

		stone_one.loggy.close()
		stone_two.loggy.close()
		with open(mutual_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@mutual_loggy-StoneOne INFO : OOH",
				"@mutual_loggy-StoneTwo INFO : EEH",
				"@mutual_loggy-StoneOne INFO : OOH",
				"@mutual_loggy-StoneTwo INFO : AH",
				"@mutual_loggy-StoneOne INFO : AH",
				"@mutual_loggy-StoneTwo INFO : TING",
				"@mutual_loggy-StoneOne INFO : TANG",
				"@mutual_loggy-StoneTwo INFO : WALLA",
				"@mutual_loggy-StoneOne INFO : WALLA",
				"@mutual_loggy-StoneTwo INFO : BING",
				"@mutual_loggy-StoneOne INFO : BANG",
			]
		)


		class logger1(LibraryContrib):

			handler			= mutual_loggy
			init_name		= "mutual_loggy_1"
			force_handover	= True

		class logger2(logger1): init_name = "mutual_loggy_2"

		loggy = []
		self.make_loggy_file(mutual_loggy)
		stone_one = StoneOne(logger1)
		stone_two = StoneTwo(logger2)

		stone_one.loggy.info("OOH")
		stone_two.loggy.info("EEH")
		stone_one.loggy.info("OOH")
		stone_two.loggy.info("AH")
		stone_one.loggy.info("AH")
		stone_two.loggy.info("TING")
		stone_one.loggy.info("TANG")
		stone_two.loggy.info("WALLA")
		stone_one.loggy.info("WALLA")
		stone_two.loggy.info("BING")
		stone_one.loggy.info("BANG")
		loggy = []

		stone_one.loggy.close()
		stone_two.loggy.close()
		with open(mutual_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@mutual_loggy_1-StoneOne INFO : OOH",
				"@mutual_loggy_2-StoneTwo INFO : EEH",
				"@mutual_loggy_1-StoneOne INFO : OOH",
				"@mutual_loggy_2-StoneTwo INFO : AH",
				"@mutual_loggy_1-StoneOne INFO : AH",
				"@mutual_loggy_2-StoneTwo INFO : TING",
				"@mutual_loggy_1-StoneOne INFO : TANG",
				"@mutual_loggy_2-StoneTwo INFO : WALLA",
				"@mutual_loggy_1-StoneOne INFO : WALLA",
				"@mutual_loggy_2-StoneTwo INFO : BING",
				"@mutual_loggy_1-StoneOne INFO : BANG",
			]
		)
		if	os.path.isfile(mutual_loggy): os.remove(mutual_loggy)








	def test_invalid_init(self):

		for item in (
			1,
			.1,
			"1",
			True,
			print,
			Transmutable,
			[ Transmutable ],
			( Transmutable, ),
			{ Transmutable },
			{ "upper_layer_link": Transmutable },
		):
			with self.subTest(upper_layer_link=item):

				pattern = f"\"{item}\" is inappropriate upper layer link"
				pattern = pattern.replace("[",r"\[").replace("]",r"\]")
				pattern = pattern.replace("(",r"\(").replace(")",r"\)")
				self.assertRaisesRegex(TypeError, pattern, Transmutable, item)




	def test_phase2raise(self):

		phase2raise = str(self.MAGICAL_ROOT /"phase2raise.loggy")
		self.make_loggy_file(phase2raise)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= phase2raise
				init_name	= "phase2raise-1"

			class Raisie(Transmutable):
				def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")
			class Rousie(Transmutable):	pass


		loggy = []
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(phase2raise) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@phase2raise-1 ERROR : Raisie nesting caused TypeError: Raisie must be stopped!",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= phase2raise
				init_name	= "phase2raise-2"
				init_level	= 10

			class Raisie(Transmutable):
				def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")
			class Rousie(Transmutable):	pass


		loggy = []
		self.make_loggy_file(phase2raise)
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(phase2raise) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@phase2raise-2-Stone ERROR : Raisie nesting caused TypeError: Raisie must be stopped!",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler			= phase2raise
				init_name		= "phase2raise-3"
				force_handover	= True

			class Raisie(Transmutable):
				def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")
			class Rousie(Transmutable):	pass


		loggy = []
		self.make_loggy_file(phase2raise)
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(phase2raise) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@phase2raise-3-Stone ERROR : Raisie nesting caused TypeError: Raisie must be stopped!",
			]
		)


		class Stone(Transmutable):
			class Raisie(Transmutable):
				def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")
			class Rousie(Transmutable):	pass


		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		if os.path.isfile(phase2raise): os.remove(phase2raise)








	def test_deep_phase2raise(self):

		deep_phase2raise = str(self.MAGICAL_ROOT /"deep_phase2raise.loggy")
		self.make_loggy_file(deep_phase2raise)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= deep_phase2raise
				init_name	= "deep_phase2raise-1"

			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")


		loggy = []
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		self.assertFalse(hasattr(self.test_case.Rousie, "Raisie"))

		self.test_case.loggy.close()
		with open(deep_phase2raise) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@deep_phase2raise-1 ERROR : Raisie nesting caused TypeError: Raisie must be stopped!",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= deep_phase2raise
				init_name	= "deep_phase2raise-2"
				init_level	= 10

			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")


		loggy = []
		self.make_loggy_file(deep_phase2raise)
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		self.assertFalse(hasattr(self.test_case.Rousie, "Raisie"))

		self.test_case.loggy.close()
		with open(deep_phase2raise) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@deep_phase2raise-2-Stone.Rousie ERROR : "
				"Raisie nesting caused TypeError: Raisie must be stopped!",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler			= deep_phase2raise
				init_name		= "deep_phase2raise-3"
				force_handover	= True

			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")


		loggy = []
		self.make_loggy_file(deep_phase2raise)
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		self.assertFalse(hasattr(self.test_case.Rousie, "Raisie"))

		self.test_case.loggy.close()
		with open(deep_phase2raise) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@deep_phase2raise-3-Stone.Rousie ERROR : "
				"Raisie nesting caused TypeError: Raisie must be stopped!",
			]
		)


		class Stone(Transmutable):
			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")


		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		self.assertFalse(hasattr(self.test_case.Rousie, "Raisie"))
		if os.path.isfile(deep_phase2raise): os.remove(deep_phase2raise)








	def test_phase2drop(self):

		phase2drop = str(self.MAGICAL_ROOT /"phase2drop.loggy")
		self.make_loggy_file(phase2drop)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= phase2drop
				init_name	= "phase2drop-1"

			class Raisie(Transmutable):
				def __new__(*args, **kwargs): pass
			class Rousie(Transmutable):	pass


		loggy = []
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(phase2drop) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@phase2drop-1 ERROR : Raisie nesting caused not Transmutable mutation",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= phase2drop
				init_name	= "phase2drop-2"
				init_level	= 10

			class Raisie(Transmutable):
				def __new__(*args, **kwargs): pass
			class Rousie(Transmutable):	pass


		loggy = []
		self.make_loggy_file(phase2drop)
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(phase2drop) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@phase2drop-2-Stone ERROR : Raisie nesting caused not Transmutable mutation",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler			= phase2drop
				init_name		= "phase2drop-3"
				force_handover	= True

			class Raisie(Transmutable):
				def __new__(*args, **kwargs): pass
			class Rousie(Transmutable):	pass


		loggy = []
		self.make_loggy_file(phase2drop)
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(phase2drop) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@phase2drop-3-Stone ERROR : Raisie nesting caused not Transmutable mutation",
			]
		)


		class Stone(Transmutable):
			class Raisie(Transmutable):
				def __new__(*args, **kwargs): pass
			class Rousie(Transmutable):	pass


		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		if os.path.isfile(phase2drop): os.remove(phase2drop)








	def test_deep_phase2drop(self):

		deep_phase2drop = str(self.MAGICAL_ROOT /"deep_phase2drop.loggy")
		self.make_loggy_file(deep_phase2drop)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= deep_phase2drop
				init_name	= "deep_phase2drop-1"

			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __new__(*args, **kwargs): pass


		loggy = []
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(deep_phase2drop) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@deep_phase2drop-1 ERROR : Raisie nesting caused not Transmutable mutation",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler		= deep_phase2drop
				init_name	= "deep_phase2drop-2"
				init_level	= 10

			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __new__(*args, **kwargs): pass


		loggy = []
		self.make_loggy_file(deep_phase2drop)
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(deep_phase2drop) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@deep_phase2drop-2-Stone.Rousie ERROR : Raisie nesting caused not Transmutable mutation",
			]
		)


		class Stone(Transmutable):
			class loggy(LibraryContrib):

				handler			= deep_phase2drop
				init_name		= "deep_phase2drop-3"
				force_handover	= True

			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __new__(*args, **kwargs): pass


		loggy = []
		self.make_loggy_file(deep_phase2drop)
		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)

		self.test_case.loggy.close()
		with open(deep_phase2drop) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@deep_phase2drop-3-Stone.Rousie ERROR : Raisie nesting caused not Transmutable mutation",
			]
		)


		class Stone(Transmutable):
			class Rousie(Transmutable):
				class Raisie(Transmutable):
					def __new__(*args, **kwargs): pass


		self.test_case = Stone()
		self.assertFalse(hasattr(self.test_case, "Raisie"))
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		if os.path.isfile(deep_phase2drop): os.remove(deep_phase2drop)
















	def test_other(self):

		class Stone(Transmutable):
			class Raisie:
				def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")
			class Rousie(Transmutable):	pass


		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		self.assertTrue(hasattr(self.test_case, "Raisie"))
		self.assertNotIsInstance(self.test_case.Raisie, Transmutable)
		self.assertIsInstance(self.test_case.Raisie, type)
		self.assertRaisesRegex(

			TypeError,
			"Raisie must be stopped",
			self.test_case.Raisie
		)


		class Stone(Transmutable):
			class Rousie(Transmutable):
				class Raisie:
					def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")


		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertIsInstance(self.test_case.Rousie, Transmutable)
		self.assertTrue(hasattr(self.test_case.Rousie, "Raisie"))
		self.assertNotIsInstance(self.test_case.Rousie.Raisie, Transmutable)
		self.assertIsInstance(self.test_case.Rousie.Raisie, type)
		self.assertRaisesRegex(

			TypeError,
			"Raisie must be stopped",
			self.test_case.Rousie.Raisie
		)








	def test_top_layer_other(self):

		class Stone:
			class Raisie:
				def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")
			class Rousie(Transmutable):	pass


		self.test_case = Stone()
		self.assertNotIsInstance(self.test_case, Transmutable)
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertNotIsInstance(self.test_case.Rousie, Transmutable)
		self.assertTrue(hasattr(self.test_case, "Raisie"))
		self.assertNotIsInstance(self.test_case.Raisie, Transmutable)
		self.assertRaisesRegex(

			TypeError,
			"Raisie must be stopped",
			self.test_case.Raisie
		)


		class Stone:
			class Rousie(Transmutable):
				class Raisie:
					def __init__(self, *args, **kwargs): raise TypeError("Raisie must be stopped!")


		self.test_case = Stone()
		self.assertNotIsInstance(self.test_case, Transmutable)
		self.assertTrue(hasattr(self.test_case, "Rousie"))
		self.assertNotIsInstance(self.test_case.Rousie, Transmutable)
		self.assertTrue(hasattr(self.test_case.Rousie, "Raisie"))
		self.assertNotIsInstance(self.test_case.Rousie.Raisie, Transmutable)
		self.assertRaisesRegex(

			TypeError,
			"Raisie must be stopped",
			self.test_case.Rousie.Raisie
		)
















	def test_kws_init(self):

		class Stone(Transmutable):
			class Measure(Transmutable):
				def weight(self): return self.W
				def height(self): return self.H


		self.test_case = Stone()
		self.assertRaisesRegex(

			AttributeError,
			"Attribute 'W' has no escalation to Stone",
			self.test_case.Measure.weight
		)
		self.assertRaisesRegex(

			AttributeError,
			"Attribute 'H' has no escalation to Stone",
			self.test_case.Measure.height
		)


		self.test_case = Stone(W=420, H=69)
		self.assertEqual(self.test_case.Measure.weight(), 420)
		self.assertEqual(self.test_case.Measure.height(), 69)


		class Stone(Transmutable):

			W = 42
			H = 667

			class Measure(Transmutable):
				def weight(self): return self.W
				def height(self): return self.H


		self.test_case = Stone()
		self.assertEqual(self.test_case.Measure.weight(), 42)
		self.assertEqual(self.test_case.Measure.height(), 667)
		self.test_case = Stone(W=420, H=69)
		self.assertEqual(self.test_case.Measure.weight(), 420)
		self.assertEqual(self.test_case.Measure.height(), 69)


		class Stone(Transmutable):
			class Measure(Transmutable):

				def weight(self): return self.W
				def height(self): return self.H
				W = 42
				H = 667


		self.test_case = Stone()
		self.assertEqual(self.test_case.Measure.weight(), 42)
		self.assertEqual(self.test_case.Measure.height(), 667)
		self.test_case = Stone(W=420, H=69)
		self.assertEqual(self.test_case.Measure.weight(), 42)
		self.assertEqual(self.test_case.Measure.height(), 667)
		self.assertEqual(self.test_case.Measure._UPPER_LAYER.W, 420)
		self.assertEqual(self.test_case.Measure._UPPER_LAYER.H, 69)
















	def test_outer_loggy(self):

		outer_loggy = str(self.MAGICAL_ROOT /"outer_loggy.loggy")
		self.make_loggy_file(outer_loggy)

		class Outdoor(Transmutable):
			class loggy(LibraryContrib):

				handler		= outer_loggy
				init_name	= "outer_loggy-1"

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(outer_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_loggy-1 INFO : I am part of mutable chain!",
				"@outer_loggy-1 INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class loggy(LibraryContrib):

				handler		= outer_loggy
				init_name	= "outer_loggy-2"
				init_level	= 10

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(outer_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_loggy-2-Stone INFO : I am part of mutable chain!",
				"@outer_loggy-2-Stone INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class loggy(LibraryContrib):

				handler			= outer_loggy
				init_name		= "outer_loggy-3"
				force_handover	= True

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(outer_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_loggy-3-Stone INFO : I am part of mutable chain!",
				"@outer_loggy-3-Stone INFO : I am root",
			]
		)
		if	os.path.isfile(outer_loggy): os.remove(outer_loggy)








	def test_outer_name_loggy(self):

		outer_name_loggy = str(self.MAGICAL_ROOT /"outer_name_loggy.loggy")
		self.make_loggy_file(outer_name_loggy)

		class Outdoor(Transmutable):
			class shmoggy(LibraryContrib):

				handler		= outer_name_loggy
				init_name	= "outer_name_loggy-1"

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(outer_name_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_name_loggy-1 INFO : I am part of mutable chain!",
				"@outer_name_loggy-1 INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class shmoggy(LibraryContrib):

				handler		= outer_name_loggy
				init_name	= "outer_name_loggy-2"
				init_level	= 10

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_name_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(outer_name_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_name_loggy-2-Stone INFO : I am part of mutable chain!",
				"@outer_name_loggy-2-Stone INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class shmoggy(LibraryContrib):

				handler			= outer_name_loggy
				init_name		= "outer_name_loggy-3"
				force_handover	= True

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_name_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(outer_name_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_name_loggy-3-Stone INFO : I am part of mutable chain!",
				"@outer_name_loggy-3-Stone INFO : I am root",
			]
		)
		if	os.path.isfile(outer_name_loggy): os.remove(outer_name_loggy)








	def test_outer_escalated_loggy(self):

		outer_e_loggy = str(self.MAGICAL_ROOT /"outer_e_loggy.loggy")
		self.make_loggy_file(outer_e_loggy)

		class Outdoor(Transmutable):
			class Handle(Transmutable):
				class loggy(LibraryContrib):

					handler		= outer_e_loggy
					init_name	= "outer_e_loggy-1"

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.test_case.loggy.info("I am root")
		self.test_case.Handle.loggy.info("I ain't root")

		self.test_case.loggy.close()
		with open(outer_e_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_e_loggy-1 INFO : I am part of mutable chain!",
				"@outer_e_loggy-1 INFO : I am root",
				"@outer_e_loggy-1 INFO : I ain't root",
			]
		)


		class Outdoor(Transmutable):
			class Handle(Transmutable):
				class loggy(LibraryContrib):

					handler		= outer_e_loggy
					init_name	= "outer_e_loggy-2"
					init_level	= 10

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_e_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.test_case.loggy.info("I am root")
		self.test_case.Handle.loggy.info("I ain't root")

		self.test_case.loggy.close()
		with open(outer_e_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_e_loggy-2-Stone INFO : I am part of mutable chain!",
				"@outer_e_loggy-2-Stone INFO : I am root",
				"@outer_e_loggy-2-Stone.Handle INFO : I ain't root",
			]
		)


		class Outdoor(Transmutable):
			class Handle(Transmutable):
				class loggy(LibraryContrib):

					handler			= outer_e_loggy
					init_name		= "outer_e_loggy-3"
					force_handover	= True

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_e_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.test_case.loggy.info("I am root")
		self.test_case.Handle.loggy.info("I ain't root")

		self.test_case.loggy.close()
		with open(outer_e_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_e_loggy-3-Stone INFO : I am part of mutable chain!",
				"@outer_e_loggy-3-Stone INFO : I am root",
				"@outer_e_loggy-3-Stone.Handle INFO : I ain't root",
			]
		)
		if	os.path.isfile(outer_e_loggy): os.remove(outer_e_loggy)








	def test_outer_escalated_name_loggy(self):

		outer_e_n_loggy = str(self.MAGICAL_ROOT /"outer_e_n_loggy.loggy")
		self.make_loggy_file(outer_e_n_loggy)

		class Outdoor(Transmutable):
			class Handle(Transmutable):
				class shmoggy(LibraryContrib):

					handler		= outer_e_n_loggy
					init_name	= "outer_e_n_loggy-1"

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.test_case.loggy.info("I am root")
		self.test_case.Handle.loggy.info("I ain't root")

		self.test_case.loggy.close()
		with open(outer_e_n_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_e_n_loggy-1 INFO : I am part of mutable chain!",
				"@outer_e_n_loggy-1 INFO : I am root",
				"@outer_e_n_loggy-1 INFO : I ain't root",
			]
		)


		class Outdoor(Transmutable):
			class Handle(Transmutable):
				class shmoggy(LibraryContrib):

					handler		= outer_e_n_loggy
					init_name	= "outer_e_n_loggy-2"
					init_level	= 10

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_e_n_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.test_case.loggy.info("I am root")
		self.test_case.Handle.loggy.info("I ain't root")

		self.test_case.loggy.close()
		with open(outer_e_n_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_e_n_loggy-2-Stone INFO : I am part of mutable chain!",
				"@outer_e_n_loggy-2-Stone INFO : I am root",
				"@outer_e_n_loggy-2-Stone.Handle INFO : I ain't root",
			]
		)


		class Outdoor(Transmutable):
			class Handle(Transmutable):
				class shmoggy(LibraryContrib):

					handler			= outer_e_n_loggy
					init_name		= "outer_e_n_loggy-3"
					force_handover	= True

		class Stone(Outdoor):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(outer_e_n_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.test_case.loggy.info("I am root")
		self.test_case.Handle.loggy.info("I ain't root")

		self.test_case.loggy.close()
		with open(outer_e_n_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@outer_e_n_loggy-3-Stone INFO : I am part of mutable chain!",
				"@outer_e_n_loggy-3-Stone INFO : I am root",
				"@outer_e_n_loggy-3-Stone.Handle INFO : I ain't root",
			]
		)
		if	os.path.isfile(outer_e_n_loggy): os.remove(outer_e_n_loggy)








	def test_outer_no_loggy(self):

		class Outdoor(Transmutable):
			def __call__(self):

				self.loggy.info("I am part of mutable chain!")
				return	"I am part of mutable chain!"

		class Stone(Outdoor):	pass

		self.test_case = Stone()
		self.assertEqual(self.test_case(), "I am part of mutable chain!")
		self.assertEqual(self.test_case.loggy.info("I am root"), NotImplemented)
















	def test_outer_nested_loggy(self):

		out_nested_loggy = str(self.MAGICAL_ROOT /"out_nested_loggy.loggy")
		self.make_loggy_file(out_nested_loggy)

		class Outdoor(Transmutable):
			class loggy(LibraryContrib):

				handler		= out_nested_loggy
				init_name	= "out_nested_loggy-1"

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(out_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_nested_loggy-1 INFO : I am part of mutable chain!",
				"@out_nested_loggy-1 INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class loggy(LibraryContrib):

				handler		= out_nested_loggy
				init_name	= "out_nested_loggy-2"
				init_level	= 10

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_nested_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(out_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_nested_loggy-2-Stone.Handle INFO : I am part of mutable chain!",
				"@out_nested_loggy-2-Stone INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class loggy(LibraryContrib):

				handler			= out_nested_loggy
				init_name		= "out_nested_loggy-3"
				force_handover	= True

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_nested_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(out_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_nested_loggy-3-Stone.Handle INFO : I am part of mutable chain!",
				"@out_nested_loggy-3-Stone INFO : I am root",
			]
		)
		if	os.path.isfile(out_nested_loggy): os.remove(out_nested_loggy)








	def test_outer_name_nested_loggy(self):

		out_n_nested_loggy = str(self.MAGICAL_ROOT /"out_n_nested_loggy.loggy")
		self.make_loggy_file(out_n_nested_loggy)

		class Outdoor(Transmutable):
			class shmoggy(LibraryContrib):

				handler		= out_n_nested_loggy
				init_name	= "out_n_nested_loggy-1"

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(out_n_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_n_nested_loggy-1 INFO : I am part of mutable chain!",
				"@out_n_nested_loggy-1 INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class shmoggy(LibraryContrib):

				handler		= out_n_nested_loggy
				init_name	= "out_n_nested_loggy-2"
				init_level	= 10

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_n_nested_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(out_n_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_n_nested_loggy-2-Stone.Handle INFO : I am part of mutable chain!",
				"@out_n_nested_loggy-2-Stone INFO : I am root",
			]
		)


		class Outdoor(Transmutable):
			class shmoggy(LibraryContrib):

				handler			= out_n_nested_loggy
				init_name		= "out_n_nested_loggy-3"
				force_handover	= True

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_n_nested_loggy)
		self.test_case = Stone()
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")

		self.test_case.loggy.close()
		with open(out_n_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_n_nested_loggy-3-Stone.Handle INFO : I am part of mutable chain!",
				"@out_n_nested_loggy-3-Stone INFO : I am root",
			]
		)
		if	os.path.isfile(out_n_nested_loggy): os.remove(out_n_nested_loggy)








	def test_outer_escalated_nested_loggy(self):

		out_e_nested_loggy = str(self.MAGICAL_ROOT /"out_e_nested_loggy.loggy")
		self.make_loggy_file(out_e_nested_loggy)

		class Outdoor(Transmutable):
			class Bound(Transmutable):
				class loggy(LibraryContrib):

					handler		= out_e_nested_loggy
					init_name	= "out_e_nested_loggy-1"

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.assertTrue(hasattr(self.test_case, "Bound"))
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		self.test_case.Bound.loggy.info("I am bound")

		self.test_case.loggy.close()
		with open(out_e_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_e_nested_loggy-1 INFO : I am part of mutable chain!",
				"@out_e_nested_loggy-1 INFO : I am root",
				"@out_e_nested_loggy-1 INFO : I am bound",
			]
		)


		class Outdoor(Transmutable):
			class Bound(Transmutable):
				class loggy(LibraryContrib):

					handler		= out_e_nested_loggy
					init_name	= "out_e_nested_loggy-2"
					init_level	= 10

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_e_nested_loggy)
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.assertTrue(hasattr(self.test_case, "Bound"))
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		self.test_case.Bound.loggy.info("I am bound")

		self.test_case.loggy.close()
		with open(out_e_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_e_nested_loggy-2-Stone.Handle INFO : I am part of mutable chain!",
				"@out_e_nested_loggy-2-Stone INFO : I am root",
				"@out_e_nested_loggy-2-Stone.Bound INFO : I am bound",
			]
		)


		class Outdoor(Transmutable):
			class Bound(Transmutable):
				class loggy(LibraryContrib):

					handler			= out_e_nested_loggy
					init_name		= "out_e_nested_loggy-3"
					force_handover	= True

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_e_nested_loggy)
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.assertTrue(hasattr(self.test_case, "Bound"))
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		self.test_case.Bound.loggy.info("I am bound")

		self.test_case.loggy.close()
		with open(out_e_nested_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_e_nested_loggy-3-Stone.Handle INFO : I am part of mutable chain!",
				"@out_e_nested_loggy-3-Stone INFO : I am root",
				"@out_e_nested_loggy-3-Stone.Bound INFO : I am bound",
			]
		)
		if	os.path.isfile(out_e_nested_loggy): os.remove(out_e_nested_loggy)








	def test_outer_escalated_name_nested_loggy(self):

		out_e_n_ne_loggy = str(self.MAGICAL_ROOT /"out_e_n_ne_loggy.loggy")
		self.make_loggy_file(out_e_n_ne_loggy)

		class Outdoor(Transmutable):
			class Bound(Transmutable):
				class shmoggy(LibraryContrib):

					handler		= out_e_n_ne_loggy
					init_name	= "out_e_n_ne_loggy-1"

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.assertTrue(hasattr(self.test_case, "Bound"))
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		self.test_case.Bound.loggy.info("I am bound")

		self.test_case.loggy.close()
		with open(out_e_n_ne_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_e_n_ne_loggy-1 INFO : I am part of mutable chain!",
				"@out_e_n_ne_loggy-1 INFO : I am root",
				"@out_e_n_ne_loggy-1 INFO : I am bound",
			]
		)


		class Outdoor(Transmutable):
			class Bound(Transmutable):
				class shmoggy(LibraryContrib):

					handler		= out_e_n_ne_loggy
					init_name	= "out_e_n_ne_loggy-2"
					init_level	= 10

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_e_n_ne_loggy)
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.assertTrue(hasattr(self.test_case, "Bound"))
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		self.test_case.Bound.loggy.info("I am bound")

		self.test_case.loggy.close()
		with open(out_e_n_ne_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_e_n_ne_loggy-2-Stone.Handle INFO : I am part of mutable chain!",
				"@out_e_n_ne_loggy-2-Stone INFO : I am root",
				"@out_e_n_ne_loggy-2-Stone.Bound INFO : I am bound",
			]
		)


		class Outdoor(Transmutable):
			class Bound(Transmutable):
				class shmoggy(LibraryContrib):

					handler			= out_e_n_ne_loggy
					init_name		= "out_e_n_ne_loggy-3"
					force_handover	= True

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		loggy = []
		self.make_loggy_file(out_e_n_ne_loggy)
		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.assertTrue(hasattr(self.test_case, "Bound"))
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.test_case.loggy.info("I am root")
		self.test_case.Bound.loggy.info("I am bound")

		self.test_case.loggy.close()
		with open(out_e_n_ne_loggy) as case_loggy:
			for line in case_loggy : loggy.append(line.rstrip("\n")[16:])

		self.assertEqual(

			loggy,
			[
				"@out_e_n_ne_loggy-3-Stone.Handle INFO : I am part of mutable chain!",
				"@out_e_n_ne_loggy-3-Stone INFO : I am root",
				"@out_e_n_ne_loggy-3-Stone.Bound INFO : I am bound",
			]
		)
		if	os.path.isfile(out_e_n_ne_loggy): os.remove(out_e_n_ne_loggy)








	def test_outer_nested_no_loggy(self):

		class Outdoor(Transmutable):
			class Bound(Transmutable):	pass

		class Stone(Outdoor):
			class Handle(Transmutable):

				def __call__(self):

					self.loggy.info("I am part of mutable chain!")
					return	"I am part of mutable chain!"


		self.test_case = Stone()
		self.assertTrue(hasattr(self.test_case, "Handle"))
		self.assertTrue(hasattr(self.test_case, "Bound"))
		self.assertEqual(self.test_case.Handle(), "I am part of mutable chain!")
		self.assertEqual(self.test_case.loggy.info("I am root"), NotImplemented)
		self.assertEqual(self.test_case.Bound.loggy.info("I am bound"), NotImplemented)








if	__name__ == "__main__" : unittest.main(verbosity=2)










import	os
import	unittest
from	time								import sleep
from	pygwarts.tests.hagrid				import EasySet
from	pygwarts.irma.contrib				import LibraryContrib
from	pygwarts.hagrid.thrivables			import Tree
from	pygwarts.hagrid.thrivables			import Copse
from	pygwarts.hagrid.sprouts				import fssprout
from	pygwarts.hagrid.planting			import Flourish
from	pygwarts.hagrid.bloom.leafs			import Rejuvenation
from	pygwarts.hagrid.planting.leafs		import LeafGrowth
from	pygwarts.hagrid.planting.peels		import GrowingPeel
from	pygwarts.hagrid.planting.peeks		import DraftPeek
from	pygwarts.hagrid.cultivation.sifting	import SiftingController








class RenewalCase(EasySet):

	"""
		Renew peeks and adding files
	"""

	@classmethod
	def tearDownClass(cls):
		cls.clean(cls)

		if	cls.clean_up:
			if	os.path.isfile(cls.EASY_HANDLER_15): os.remove(cls.EASY_HANDLER_15)

	@classmethod
	def setUpClass(cls):

		super().setUpClass()
		cls.make_loggy_file(cls, cls.EASY_HANDLER_15)

		cls.fmake(cls, cls.dst_pros_file2)
		cls.fmake(cls, cls.dst_pros_file3)
		cls.fmake(cls, cls.dst_cons_file1)
		cls.fmake(cls, cls.dst_cons_file2)


	def test_1_first_flourish(self):
		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.EASY_HANDLER_15
				init_name	= "1_first_flourish"
				init_level	= 10

			class Renewal(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=True)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass

			class Adding(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=False)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass
				class leafs(SiftingController):

					include	= "",

			@fssprout(self.EASY_SET_SPROUT)
			class sync(Flourish):			pass

		self.assertFalse(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		with self.assertLogs("1_first_flourish", 10) as case_loggy:

			self.test_case = Forest()
			self.test_case.sync()

		self.no_loggy_levels(case_loggy.output, 30,40,50)

		self.assertFalse(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_pros_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_pros_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_pros_file3}\"", case_loggy.output)
		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_cons_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_cons_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_redundant_1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:1_first_flourish:Grown leaf \"{self.dst_redundant_2}\"", case_loggy.output)








	def test_2_touch_flourish(self):
		sleep(1.1)

		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.EASY_HANDLER_15
				init_name	= "2_touch_flourish"
				init_level	= 10

			class Renewal(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=True)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass

			class Adding(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=False)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass
				class leafs(SiftingController):

					include	= "",

			@fssprout(self.EASY_SET_SPROUT)
			class sync(Flourish):			pass

		self.assertFalse(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.fmake(self.pros_file2)
		self.fmake(self.cons_file1)

		with self.assertLogs("2_touch_flourish", 10) as case_loggy:

			self.test_case = Forest()
			self.test_case.sync()

		self.no_loggy_levels(case_loggy.output, 30,40,50)

		self.assertFalse(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.assertNotIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_pros_file1}\"", case_loggy.output)
		self.assertIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_pros_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_pros_file3}\"", case_loggy.output)
		self.assertIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_cons_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_cons_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_redundant_1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_redundant_2}\"", case_loggy.output)

		self.assertEqual(
			case_loggy.output.count(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_pros_file2}\""),1
		)
		self.assertEqual(
			case_loggy.output.count(f"INFO:2_touch_flourish:Grown leaf \"{self.dst_cons_file1}\""),1
		)








	def test_3_no_touch_flourish(self):
		sleep(1.1)

		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.EASY_HANDLER_15
				init_name	= "3_no_touch_flourish"
				init_level	= 10

			class Renewal(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=True)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass

			class Adding(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=False)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass
				class leafs(SiftingController):

					include	= "",

			@fssprout(self.EASY_SET_SPROUT)
			class sync(Flourish):			pass

		self.assertFalse(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		with self.assertLogs("3_no_touch_flourish", 10) as case_loggy:

			self.test_case = Forest()
			self.test_case.sync()

		self.no_loggy_levels(case_loggy.output, 30,40,50)

		self.assertFalse(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_pros_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_pros_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_pros_file3}\"", case_loggy.output)
		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_cons_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_cons_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_redundant_1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:3_no_touch_flourish:Grown leaf \"{self.dst_redundant_2}\"", case_loggy.output)








	def test_4_add_flourish(self):
		sleep(1.1)

		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.EASY_HANDLER_15
				init_name	= "4_add_flourish"
				init_level	= 10

			class Renewal(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=True)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass

			class Adding(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=False)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass
				class leafs(SiftingController):

					include	= self.file1.replace("\\", "\\\\"),

			@fssprout(self.EASY_SET_SPROUT)
			class sync(Flourish):			pass

		self.assertFalse(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		with self.assertLogs("4_add_flourish", 10) as case_loggy:

			self.test_case = Forest()
			self.test_case.sync()

		self.no_loggy_levels(case_loggy.output, 30,40,50)

		self.assertTrue(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertTrue(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.assertIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_pros_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_pros_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_pros_file3}\"", case_loggy.output)
		self.assertNotIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_cons_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_cons_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_redundant_1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:4_add_flourish:Grown leaf \"{self.dst_redundant_2}\"", case_loggy.output)

		self.assertEqual(
			case_loggy.output.count(f"INFO:4_add_flourish:Grown leaf \"{self.dst_file1}\""),1
		)








	def test_5_remove_flourish(self):
		sleep(1.1)

		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.EASY_HANDLER_15
				init_name	= "5_remove_flourish"
				init_level	= 10

			class Renewal(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=True)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass

			class Adding(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=False)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass
				class leafs(SiftingController):

					include	= self.file1.replace("\\", "\\\\"),

			@fssprout(self.EASY_SET_SPROUT)
			class sync(Flourish):			pass

		os.remove(self.dst_pros_file2)
		self.assertTrue(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertFalse(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		with self.assertLogs("5_remove_flourish", 10) as case_loggy:

			self.test_case = Forest()
			self.test_case.sync()

		self.no_loggy_levels(case_loggy.output, 30,40,50)

		self.assertTrue(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertFalse(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_pros_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_pros_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_pros_file3}\"", case_loggy.output)
		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_cons_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_cons_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_redundant_1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:5_remove_flourish:Grown leaf \"{self.dst_redundant_2}\"", case_loggy.output)








	def test_6_touch_flourish(self):
		sleep(1.1)

		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.EASY_HANDLER_15
				init_name	= "6_touch_flourish"
				init_level	= 10

			class Adding(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=False)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass
				class leafs(SiftingController):

					include	= self.file1.replace("\\", "\\\\"),

			class Renewal(Tree):

				bough = self.EASY_SET_BOUGH
				@GrowingPeel
				@DraftPeek(renew=True)
				class grow(LeafGrowth):		pass
				class files(Rejuvenation):	pass

			@fssprout(self.EASY_SET_SPROUT)
			class sync(Flourish):			pass

		self.assertTrue(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertFalse(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.fmake(self.file1)
		self.fmake(self.pros_file1)
		self.fmake(self.pros_file2)
		self.fmake(self.pros_file3)
		self.fmake(self.cons_file1)
		self.fmake(self.cons_file2)
		self.fmake(self.redundant_1)
		self.fmake(self.redundant_2)

		with self.assertLogs("6_touch_flourish", 10) as case_loggy:

			self.test_case = Forest()
			self.test_case.sync()

		self.no_loggy_levels(case_loggy.output, 30,40,50)

		self.assertTrue(os.path.isfile(self.dst_file1))
		self.assertTrue(os.path.isdir(self.dst_pros_folder))
		self.assertFalse(os.path.isfile(self.dst_pros_file1))
		self.assertFalse(os.path.isfile(self.dst_pros_file2))
		self.assertTrue(os.path.isfile(self.dst_pros_file3))
		self.assertTrue(os.path.isdir(self.dst_cons_folder))
		self.assertTrue(os.path.isfile(self.dst_cons_file1))
		self.assertTrue(os.path.isfile(self.dst_cons_file2))
		self.assertFalse(os.path.isdir(self.dst_redundant_1_folder))
		self.assertFalse(os.path.isdir(self.dst_redundant_2_folder))
		self.assertFalse(os.path.isfile(self.dst_redundant_1))
		self.assertFalse(os.path.isfile(self.dst_redundant_2))

		self.assertIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_pros_file1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_pros_file2}\"", case_loggy.output)
		self.assertIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_pros_file3}\"", case_loggy.output)
		self.assertIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_cons_file1}\"", case_loggy.output)
		self.assertIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_cons_file2}\"", case_loggy.output)
		self.assertNotIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_redundant_1}\"", case_loggy.output)
		self.assertNotIn(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_redundant_2}\"", case_loggy.output)

		self.assertEqual(
			case_loggy.output.count(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_file1}\"",), 1
		)
		self.assertEqual(
			case_loggy.output.count(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_pros_file3}\"",), 1
		)
		self.assertEqual(
			case_loggy.output.count(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_cons_file1}\"",), 1
		)
		self.assertEqual(
			case_loggy.output.count(f"INFO:6_touch_flourish:Grown leaf \"{self.dst_cons_file2}\"",), 1
		)








if	__name__ == "__main__" : unittest.main(verbosity=2)










import	re
from	typing							import Literal
from	pygwarts.magical.spells			import flagrate
from	pygwarts.irma.access.volume		import LibraryVolume
from	pygwarts.irma.access.handlers	import AccessHandler








class GroupParser(AccessHandler):

	"""
		Super implementation of handler, that suggests parsing the line it receives. Parsing occurs with
		the "rpattern" filed, which must be a valid regular expression or a compilable string. Basically
		this super is just for validating "rpattern" field. If it is neither a string nor a Pattern,
		corresponding warning message will be logged and "rpattern" will stay as it is.
	"""

	rpattern :str | re.Pattern

	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)


		match getattr(self, "rpattern", None):
			case str():

				self.rpattern = re.compile(self.rpattern)
				self.loggy.debug(f"Accepted string \"{self.rpattern.pattern}\"")

			case re.Pattern():	self.loggy.debug(f"Accepted pattern \"{self.rpattern.pattern}\"")
			case None:			self.loggy.warning(f"{self} doesn't have pattern for parsing")
			case _:				self.loggy.warning(f"{self} has invalid pattern for parsing")








class TargetHandler(GroupParser):

	"""
		implementation of handler, that suggests parsing single value from a line it receives, by
		the name group "target". It is implies, that "rpattern" regex must contain such name group
		"target" that must encompass desired text to catch. In case of successful parse, assigns
		caught "target" string value as it's "recap" mapping in a LibraryVolume "volume" object and
		returns True. It must be noticed, that multiple parsing will lead to reassign of "recap"
		mapping. Returns None in any other case. Doesn't handle possible regex search Exception.
	"""

	def __call__(self, line :str, volume :LibraryVolume) -> Literal[True] | None :

		if	isinstance(getattr(self, "rpattern", None), re.Pattern):
			if	(match := self.rpattern.search(line)) and (target := match.group("target")):
				if	self.registered(volume):


					volume[self]["recap"] = target
					self.loggy.debug(f"Recap set to \"{target}\"")


					return True
			else:	self.loggy.debug("No match for target")








class TargetNumberAccumulator(GroupParser):

	"""
		implementation of handler, that suggests parsing single numeric value from a line it receives, by
		the name group "target". It is implies, that "rpattern" regex must contain such name group
		"target" that must encompass desired numerical text to catch. In case of successful parse,
		adds caught "target" numerical value to it's "recap" mapping in a LibraryVolume "volume" object,
		which by default will be set to 0, and returns True. Returns None in any other case. Doesn't handle
		possible regex search Exception.
	"""

	def __call__(self, line :str, volume :LibraryVolume) -> Literal[True] | None :

		if	isinstance(getattr(self, "rpattern", None), re.Pattern):
			if	(match := self.rpattern.search(line)) and (target := match.group("target")):
				if	self.registered(volume):


					current = volume[self]["recap"] = eval(f"volume[self].setdefault('recap',0) +{target}")
					self.loggy.debug(f"Recap accumulation is \"{current}\"")


					return True
			else:	self.loggy.debug("No match for target")








class TargetStringAccumulator(GroupParser):

	"""
		implementation of handler, that suggests parsing single value from a line it receives, by
		the name group "target". It is implies, that "rpattern" regex must contain such name group
		"target" that must encompass desired text to catch. In case of successful parse, adds
		caught "target" string value to it's "recap" mapping in a LibraryVolume "volume" object, which
		by default will be set to a list, and returns True. Returns None in any other case. Doesn't
		handle possible regex search Exception, or situation when "recap" mapping already exist and
		is not a list.
	"""

	def __call__(self, line :str, volume :LibraryVolume) -> Literal[True] | None :

		if	isinstance(getattr(self, "rpattern", None), re.Pattern):
			if	(match := self.rpattern.search(line)) and (target := match.group("target")):
				if	self.registered(volume):


					volume[self].setdefault("recap",[]).append(target)
					rl = len(volume[self]["recap"])
					self.loggy.debug(f"Recap extended to {rl} item{flagrate(rl)}")


					return True
			else:	self.loggy.debug("No match for target")










import	os
import	unittest
from	time								import sleep
from	pygwarts.tests.hagrid				import MediumSet
from	pygwarts.irma.contrib				import LibraryContrib
from	pygwarts.hagrid.thrivables			import Tree
from	pygwarts.hagrid.thrivables			import Copse
from	pygwarts.hagrid.sprouts				import fssprout
from	pygwarts.hagrid.planting			import Flourish
from	pygwarts.hagrid.bloom.leafs			import Rejuvenation
from	pygwarts.hagrid.planting.leafs		import LeafMove
from	pygwarts.hagrid.cultivation.sifting	import SiftingController








class SortingCase(MediumSet):

	""" Sorting of files. From source directly to the bough. """

	@classmethod
	def tearDownClass(cls):
		cls.clean(cls)

		if	cls.clean_up:
			if	os.path.isfile(cls.MEDIUM_HANDLER_11): os.remove(cls.MEDIUM_HANDLER_11)

	@classmethod
	def setUpClass(cls):

		super().setUpClass()
		cls.make_loggy_file(cls, cls.MEDIUM_HANDLER_11)

		cls.f_mss1_resume		= os.path.join(cls.MEDIUM_SET_SPROUT_1, "resume.docx")
		cls.f_mss1_md_grocery	= os.path.join(cls.d_mss1_md, "grocery.txt")
		cls.f_mss1_md_gc_ps		= os.path.join(cls.d_mss1_md_gc, "ps i love you.avi")
		cls.f_mss1_md_gc_h		= os.path.join(cls.d_mss1_md_gc, "something about hope.mkv")
		cls.f_mss1_c_f1			= os.path.join(cls.d_mss1_c, "foto.png")
		cls.f_mss1_c_f2			= os.path.join(cls.d_mss1_c, "foto.jpeg")

		cls.f_mss2_vinstr		= os.path.join(cls.MEDIUM_SET_SPROUT_2, "vacuum cleaner.pdf")
		cls.f_mss2_c_summer		= os.path.join(cls.d_mss2_c, "summer.jpg")
		cls.f_mss2_c_winter		= os.path.join(cls.d_mss2_c, "winter.jpg")
		cls.f_mss2_c_autumn		= os.path.join(cls.d_mss2_c, "autumn.jpg")
		cls.f_mss2_c_spring		= os.path.join(cls.d_mss2_c, "spring.jpg")

		cls.f_mss3_k_fdesc		= os.path.join(cls.d_mss3_k, "fridge.pdf")
		cls.f_mss3_k_odesc		= os.path.join(cls.d_mss3_k, "oven.pdf")
		cls.f_mss3_k_t_crumf1	= os.path.join(cls.d_mss3_k_t, "crumb.png")
		cls.f_mss3_k_t_crumf2	= os.path.join(cls.d_mss3_k_t, "crumb2.png")
		cls.f_mss3_k_t_crumv1	= os.path.join(cls.d_mss3_k_t, "holy crumb.mp4")
		cls.f_mss3_k_t_crumv2	= os.path.join(cls.d_mss3_k_t, "holy crumb2.mp4")

		cls.fmake(cls, cls.f_mss1_resume, "ready to work!")
		cls.fmake(cls, cls.f_mss1_md_grocery, "milk, meat, bread")
		cls.fmake(cls, cls.f_mss1_md_gc_ps, "she might not love it")
		cls.fmake(cls, cls.f_mss1_md_gc_h, "can't even recall name")
		cls.fmake(cls, cls.f_mss1_c_f1, "me")
		cls.fmake(cls, cls.f_mss1_c_f2, "not me")

		cls.fmake(cls, cls.f_mss2_vinstr, "how to use it")
		cls.fmake(cls, cls.f_mss2_c_summer, "summer")
		cls.fmake(cls, cls.f_mss2_c_winter, "winter")
		cls.fmake(cls, cls.f_mss2_c_autumn, "autumn")
		cls.fmake(cls, cls.f_mss2_c_spring, "spring")

		cls.fmake(cls, cls.f_mss3_k_fdesc, "the fridge")
		cls.fmake(cls, cls.f_mss3_k_odesc, "the oven")
		cls.fmake(cls, cls.f_mss3_k_t_crumf1, "crumb1 is lit")
		cls.fmake(cls, cls.f_mss3_k_t_crumf2, "crumb2 is litter")
		cls.fmake(cls, cls.f_mss3_k_t_crumv1, "why lit")
		cls.fmake(cls, cls.f_mss3_k_t_crumv2, "why litter")

		cls.tg1_f_mss1_resume		= os.path.join(cls.MEDIUM_SET_BOUGH_1, "resume.docx")
		cls.tg1_f_mss1_md_grocery	= os.path.join(cls.MEDIUM_SET_BOUGH_1, "grocery.txt")
		cls.tg1_f_mss2_vinstr		= os.path.join(cls.MEDIUM_SET_BOUGH_1, "vacuum cleaner.pdf")
		cls.tg1_f_mss3_k_fdesc		= os.path.join(cls.MEDIUM_SET_BOUGH_1, "fridge.pdf")
		cls.tg1_f_mss3_k_odesc		= os.path.join(cls.MEDIUM_SET_BOUGH_1, "oven.pdf")

		cls.tg2_f_mss1_md_gc_ps		= os.path.join(cls.MEDIUM_SET_BOUGH_3, "ps i love you.avi")
		cls.tg2_f_mss1_md_gc_h		= os.path.join(cls.MEDIUM_SET_BOUGH_3, "something about hope.mkv")
		cls.tg2_f_mss3_k_t_crumv1	= os.path.join(cls.MEDIUM_SET_BOUGH_3, "holy crumb.mp4")
		cls.tg2_f_mss3_k_t_crumv2	= os.path.join(cls.MEDIUM_SET_BOUGH_3, "holy crumb2.mp4")

		cls.tg3_f_mss2_c_summer		= os.path.join(cls.MEDIUM_SET_BOUGH_2, "summer.jpg")
		cls.tg3_f_mss2_c_winter		= os.path.join(cls.MEDIUM_SET_BOUGH_2, "winter.jpg")
		cls.tg3_f_mss2_c_autumn		= os.path.join(cls.MEDIUM_SET_BOUGH_2, "autumn.jpg")
		cls.tg3_f_mss2_c_spring		= os.path.join(cls.MEDIUM_SET_BOUGH_2, "spring.jpg")
		cls.tg3_f_mss1_c_f1			= os.path.join(cls.MEDIUM_SET_BOUGH_2, "foto.png")
		cls.tg3_f_mss1_c_f2			= os.path.join(cls.MEDIUM_SET_BOUGH_2, "foto.jpeg")
		cls.tg3_f_mss3_k_t_crumf1	= os.path.join(cls.MEDIUM_SET_BOUGH_2, "crumb.png")
		cls.tg3_f_mss3_k_t_crumf2	= os.path.join(cls.MEDIUM_SET_BOUGH_2, "crumb2.png")

	def setUp(self):
		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.MEDIUM_HANDLER_11
				init_level	= 10

			class First(Tree):

				bough = self.MEDIUM_SET_BOUGH_1
				class leafs(SiftingController): include = r".+\.(docx?|txt|pdf)$",

			class Second(Tree):

				bough = self.MEDIUM_SET_BOUGH_2
				class leafs(SiftingController): include = r".+\.(avi|mkv|mp4)$",

			class Third(Tree):

				bough = self.MEDIUM_SET_BOUGH_3
				class leafs(SiftingController): include = r".+\.(jpe?g|png|gif)$",

			class grow(LeafMove):		pass
			class files(Rejuvenation):	pass

			@fssprout(self.MEDIUM_SET_SPROUT_3)	# Order does matter cause every sprout pack it'self
			@fssprout(self.MEDIUM_SET_SPROUT_2)	# before previous sprout, so the nearest to dispatcher
			@fssprout(self.MEDIUM_SET_SPROUT_1)	# sprout will be processed first.
			class sync(Flourish):		pass


		self.test_case = Forest




	def test_first_flourish(self):

		self.test_case.loggy.init_name = "first_flourish"
		self.assertTrue(os.path.isdir(self.MEDIUM_SET_BOUGH_1))
		self.assertTrue(os.path.isdir(self.MEDIUM_SET_BOUGH_2))
		self.assertTrue(os.path.isdir(self.MEDIUM_SET_BOUGH_3))


		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		self.assertTrue(os.path.isfile(self.f_mss1_resume))
		self.assertTrue(os.path.isfile(self.f_mss1_md_grocery))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_ps))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_h))
		self.assertTrue(os.path.isfile(self.f_mss1_c_f1))
		self.assertTrue(os.path.isfile(self.f_mss1_c_f2))

		# Second sprout
		self.assertTrue(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		self.assertTrue(os.path.isfile(self.f_mss2_vinstr))
		self.assertTrue(os.path.isfile(self.f_mss2_c_summer))
		self.assertTrue(os.path.isfile(self.f_mss2_c_winter))
		self.assertTrue(os.path.isfile(self.f_mss2_c_autumn))
		self.assertTrue(os.path.isfile(self.f_mss2_c_spring))

		# Third sprout
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))

		self.assertTrue(os.path.isfile(self.f_mss3_k_fdesc))
		self.assertTrue(os.path.isfile(self.f_mss3_k_odesc))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_crumf1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_crumf2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_crumv1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_crumv2))

		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("first_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)




		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		self.assertFalse(os.path.isfile(self.f_mss1_resume))
		self.assertFalse(os.path.isfile(self.f_mss1_md_grocery))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_ps))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_h))
		self.assertFalse(os.path.isfile(self.f_mss1_c_f1))
		self.assertFalse(os.path.isfile(self.f_mss1_c_f2))

		# Second sprout
		self.assertFalse(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		self.assertFalse(os.path.isfile(self.f_mss2_vinstr))
		self.assertFalse(os.path.isfile(self.f_mss2_c_summer))
		self.assertFalse(os.path.isfile(self.f_mss2_c_winter))
		self.assertFalse(os.path.isfile(self.f_mss2_c_autumn))
		self.assertFalse(os.path.isfile(self.f_mss2_c_spring))

		# Third sprout
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))

		self.assertFalse(os.path.isfile(self.f_mss3_k_fdesc))
		self.assertFalse(os.path.isfile(self.f_mss3_k_odesc))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumf1))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumf2))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumv1))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumv2))

		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_resume))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_grocery))
		self.assertTrue(os.path.isfile(self.tg1_f_mss2_vinstr))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_fdesc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_odesc))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		self.assertTrue(self.tg2_f_mss1_md_gc_ps)
		self.assertTrue(self.tg2_f_mss1_md_gc_h)
		self.assertTrue(self.tg2_f_mss3_k_t_crumv1)
		self.assertTrue(self.tg2_f_mss3_k_t_crumv2)
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))
		self.assertTrue(self.tg3_f_mss2_c_summer)
		self.assertTrue(self.tg3_f_mss2_c_winter)
		self.assertTrue(self.tg3_f_mss2_c_autumn)
		self.assertTrue(self.tg3_f_mss2_c_spring)
		self.assertTrue(self.tg3_f_mss1_c_f1)
		self.assertTrue(self.tg3_f_mss1_c_f2)
		self.assertTrue(self.tg3_f_mss3_k_t_crumf1)
		self.assertTrue(self.tg3_f_mss3_k_t_crumf2)




		# First sprout first bough
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_md))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_gc_onion))


		# Second sprout first bough
		self.assertFalse(os.path.isdir(self.tg1_d_mss2_c))
		self.assertTrue(os.path.isfile(self.tg1_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyphilosophizing))


		# Third sprout first bough
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_f))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb69))




		# First sprout second bough
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_gc_onion))

		# Second sprout second bough
		self.assertFalse(os.path.isdir(self.tg2_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_easyphilosophizing))

		# Third sprout second bough
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_f))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb69))




		# First sprout third bough
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_onion))

		# Second sprout third bough
		self.assertFalse(os.path.isdir(self.tg3_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.tg3_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyphilosophizing))

		# Third sprout third bough
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_f))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb69))
















	def test_no_touch_flourish(self):

		sleep(1.1)
		self.test_case.loggy.init_name = "no_touch_flourish"


	# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		self.assertFalse(os.path.isfile(self.f_mss1_resume))
		self.assertFalse(os.path.isfile(self.f_mss1_md_grocery))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_ps))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_h))
		self.assertFalse(os.path.isfile(self.f_mss1_c_f1))
		self.assertFalse(os.path.isfile(self.f_mss1_c_f2))

		# Second sprout
		self.assertFalse(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		self.assertFalse(os.path.isfile(self.f_mss2_vinstr))
		self.assertFalse(os.path.isfile(self.f_mss2_c_summer))
		self.assertFalse(os.path.isfile(self.f_mss2_c_winter))
		self.assertFalse(os.path.isfile(self.f_mss2_c_autumn))
		self.assertFalse(os.path.isfile(self.f_mss2_c_spring))

		# Third sprout
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))

		self.assertFalse(os.path.isfile(self.f_mss3_k_fdesc))
		self.assertFalse(os.path.isfile(self.f_mss3_k_odesc))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumf1))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumf2))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumv1))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumv2))

		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("no_touch_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)




		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		self.assertFalse(os.path.isfile(self.f_mss1_resume))
		self.assertFalse(os.path.isfile(self.f_mss1_md_grocery))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_ps))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_h))
		self.assertFalse(os.path.isfile(self.f_mss1_c_f1))
		self.assertFalse(os.path.isfile(self.f_mss1_c_f2))

		# Second sprout
		self.assertFalse(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		self.assertFalse(os.path.isfile(self.f_mss2_vinstr))
		self.assertFalse(os.path.isfile(self.f_mss2_c_summer))
		self.assertFalse(os.path.isfile(self.f_mss2_c_winter))
		self.assertFalse(os.path.isfile(self.f_mss2_c_autumn))
		self.assertFalse(os.path.isfile(self.f_mss2_c_spring))

		# Third sprout
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))

		self.assertFalse(os.path.isfile(self.f_mss3_k_fdesc))
		self.assertFalse(os.path.isfile(self.f_mss3_k_odesc))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumf1))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumf2))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumv1))
		self.assertFalse(os.path.isfile(self.f_mss3_k_t_crumv2))

		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_resume))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_grocery))
		self.assertTrue(os.path.isfile(self.tg1_f_mss2_vinstr))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_fdesc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_odesc))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		self.assertTrue(self.tg2_f_mss1_md_gc_ps)
		self.assertTrue(self.tg2_f_mss1_md_gc_h)
		self.assertTrue(self.tg2_f_mss3_k_t_crumv1)
		self.assertTrue(self.tg2_f_mss3_k_t_crumv2)
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))
		self.assertTrue(self.tg3_f_mss2_c_summer)
		self.assertTrue(self.tg3_f_mss2_c_winter)
		self.assertTrue(self.tg3_f_mss2_c_autumn)
		self.assertTrue(self.tg3_f_mss2_c_spring)
		self.assertTrue(self.tg3_f_mss1_c_f1)
		self.assertTrue(self.tg3_f_mss1_c_f2)
		self.assertTrue(self.tg3_f_mss3_k_t_crumf1)
		self.assertTrue(self.tg3_f_mss3_k_t_crumf2)




		# First sprout first bough
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_md))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_gc_onion))


		# Second sprout first bough
		self.assertFalse(os.path.isdir(self.tg1_d_mss2_c))
		self.assertTrue(os.path.isfile(self.tg1_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyphilosophizing))


		# Third sprout first bough
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_f))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb69))




		# First sprout second bough
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_gc_onion))

		# Second sprout second bough
		self.assertFalse(os.path.isdir(self.tg2_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg2_f_mss2_c_easyphilosophizing))

		# Third sprout second bough
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_f))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg2_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg2_f_mss3_k_t_bc_crumb69))




		# First sprout third bough
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_onion))

		# Second sprout third bough
		self.assertFalse(os.path.isdir(self.tg3_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.tg3_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyphilosophizing))

		# Third sprout third bough
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_f))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb69))








if	__name__ == "__main__" : unittest.main(verbosity=2)










import	os
import	sys
import	argparse
import	unittest
from	shutil			import rmtree
from	pygwarts.tests	import PygwartsTestCase








if	__name__ == "__main__" :

	current_test = argparse.ArgumentParser(

		prog="unit_main",
		usage="unit_main [-i {magical, irma, hagrid, filch, hedwig}] [-d PATH] [-c]",
		description=str(

			"pygwarts unit tests module.\n"
			"Current tests count:\n"
			"total:   1099\n"
			"magical: 455\n"
			"hagrid:  240\n"
			"irma:    235\n"
			"filch:   96\n"
			"hedwig:  73\n"
			"\n"
		),
		epilog=str(

			"pygwarts 1.4.9.6"
			"\nlngd\n\n"
		),
		formatter_class=argparse.RawDescriptionHelpFormatter
	)
	current_test.add_argument(

		"-i", "--include",
		help="include certain modules to run unit tests (all included if omitted)",
		default=[ "magical", "irma", "hagrid", "filch", "hedwig" ],
		nargs="+",
	)
	current_test.add_argument(

		"-d", "--directory",
		help=str(

			"set working directory for tests files "
			f"(default {os.path.expanduser('~')}{os.path.sep}pygwarts-test-folder)"
		),
	)
	current_test.add_argument(

		"-c", "--clean-up",
		action="store_true",
		help=str("erase working directory after tests run, no matter results"),
	)
	current_args = current_test.parse_args()


	if current_args.directory is not None : PygwartsTestCase._PygwartsTestCase__CWD = current_args.directory
	PygwartsTestCase.clean_up = current_args.clean_up


	for module_name in set(current_args.include):
		match module_name:

			case "magical":	from pygwarts.tests.magical.unit_magical_main	import *
			case "irma":	from pygwarts.tests.irma.unit_irma_main			import *
			case "hagrid":	from pygwarts.tests.hagrid.unit_hagrid_main		import *
			case "filch":	from pygwarts.tests.filch.unit_filch_main		import *
			case "hedwig":	from pygwarts.tests.hedwig.unit_hedwig_main		import *
			case _:

				print(f"\nInvalid module \"{module_name}\"")
				sys.exit(1)


	unittest.main(verbosity=2, argv=[ sys.argv[0] ], exit=False)
	if current_args.clean_up : rmtree(PygwartsTestCase._PygwartsTestCase__CWD)










from os													import access	as osaccess
from os													import path		as ospath
from os													import R_OK
from typing												import Callable
from pygwarts.magical.philosophers_stone				import Transmutable
from pygwarts.magical.philosophers_stone.transmutations	import ControlledTransmutation
from pygwarts.magical.spells							import geminio
from pygwarts.hagrid.walks								import fstree








class fssprout(ControlledTransmutation):

	"""
		hagrid core functionality decorator, that serves as a planting dispatching automation for a
		certain root folder.
		As ControlledTransmutation class, accepts single "sprout" argument, that must be only a string
		that represents source root folder absolute path. The "sprout" for hagrid is the only path, that
		always stays as a string, cause it's role is only a starter, so no Path object conversion needed and
		all "planting" organization relies on a "sprout" to be a string.
		In mutable chain acts as a mutation - takes decorated planting dispatching class and extends it by
		declaring meta __call__ to invoke decorated __call__. Meta __call__ will "walk" from that "sprout".
		That means by the use of "fstree" function will be created generator object, that will traverse the
		file system tree, and, starting from provided "sprout" folder and for every subfolder, that generator
		will yield a tuple of three objects:
			branch	- current folder walked;
			twigs	- list of Path objects, that represents all folders in current folder;
			leafs	- list of Path objects, that represents all files in current folder.
		This tuple preceded with current sprout considered as a "plant" for a decorated dispatcher.
		This decorator implemented the way it is possible to use multiple such decorators, decorating each
		other. The Meta __call__ for each decorator will consider *args and **kwargs from decorated __call__
		as possible "plants" and pass it to decorated __call__ right after itself generator exhaust. Even if
		its own "plant" failed, all arguments will be passed to decorated object. That allows scheduling
		valid sprouts, bypassing failed.
	"""


	def __init__(self, sprout :str):
		self.sprout = sprout

	def _mutable_chain_injection(self, mutable_layer :Transmutable) -> Transmutable :

		sprout	= self.sprout
		reason	= None
		plan	= None


		if	isinstance(self.sprout, str):
			if	ospath.isdir(self.sprout):
				if	osaccess(self.sprout, R_OK): plan = fstree(sprout)


				else:	reason = f"Sprout \"{sprout}\" is inaccessible"
			else:		reason = f"Sprout \"{sprout}\" is not thrived"
		else:			reason = f"Sprout \"{sprout}\" is invalid"


		class SproutGenerator(geminio(mutable_layer)):
			def __call__(self, *args, **kwargs):


				if	plan is not None:
					for	plant in plan:


						if	plant : super().__call__(sprout, *plant)
				elif	isinstance(reason, str): self.loggy.critical(reason)


				# After current generator will exhaust, right after this and at every consecutive call to
				# current sprout layer "args" will be considered as upper layer decorator plants and will
				# be delegated to decorated layer by __call__, along with any provided key-word flags.
				super().__call__(*args, **kwargs)
		return	SproutGenerator







